title = "Transkrypcja audio"
menu_header: str = "# Menu"
files_label: str = "Pliki do transkrypcji:"
browse_files_btn: str = "Wybierz pliki"
dir_label: str = "Folder do zapisu:"
browse_dir_btn: str = "Wybierz folder"
model_dropdown_label: str = "Model"
transcribe_btn: str = "Wykonaj transkrypcję"
results_header: str = "# Wyniki"
results_table_header: list[str] = ["Plik audio", "Plik tekstowy"]
loading_model: str = "Trwa ładowanie modelu:"
loaded_model: str = "Załadowano model:"
file_not_exist: str = "Plik nie istnieje."
processing_file: str = "Trwa przetwarzanie pliku:"
processing_file_ended: str = "Zakończono przetwarzanie pliku:"
processing_file_error: str = "Wystąpił błąd podczas transkrypcji."
saving_file_error: str = "Wystąpił błąd podczas zapisu do pliku."
